import java.awt.Point;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

public class Connection extends Thread {
	ObjectInputStream inStream;
	ObjectOutputStream outStream;
	Socket inSocket;
	Socket outSocket;
	String name;
	boolean ready = false;

	public Connection(Socket inSocket) throws IOException, ClassNotFoundException {
		printScreen("Verbindungsaufbau mit: " + inSocket.getInetAddress());
		inStream = new ObjectInputStream(inSocket.getInputStream());
		this.inSocket = inSocket;
		name = receiveString();
		printScreen("InputStream mit " + name + " aufgebaut");
	}
	
	public void setOutSocket(Socket outSocket) throws IOException {
		this.outSocket = outSocket;
		outStream = new ObjectOutputStream(outSocket.getOutputStream());
		printScreen("OutputStream von " + name + " wurde mit InputStream von " + MainClass.getOppositeConnection(this).name + " verknpft.");
	}

	@Override
	public void run() {
		printScreen("Thread " + name + " gestartet");				
		boolean connected = true;
		while (connected) {
			try {
				Command in = receiveCommand();
				if (in.result == Command.results.ready) {
					ready = true;
					if (MainClass.getOppositeConnection(this).ready) {
						sendCommand(new Command(Command.modes.info, Command.results.gameStart, new Point()));
						MainClass.getOppositeConnection(this).sendCommand(new Command(Command.modes.info, Command.results.gameStart, new Point()));
					}
				} else {
					sendCommand(in);					
				}
				
			} catch (Exception e) {
				printError(e.toString());
				connected = false;
			}
		}
		try {
			printScreen(name + " disconnected");
			inSocket.close();
			outSocket.close();
		} catch (IOException e) {
			printScreen(e.toString());
		} finally {
			printScreen(name + " removed");
			MainClass.conList.remove(this);
			try {
				MainClass.conList.remove(MainClass.getOppositeConnection(this));
			} catch (Exception ex) {
			}
		}
	}

	private String receiveString() throws ClassNotFoundException, IOException {
		String o = (String) inStream.readObject();
		printScreen(o.toString(), "-->");
		return o;
	}

	public void sendGameInfo(GameInfo o) throws IOException {
		outStream.writeObject(o);
		printScreen(o.toString(), "<--");
	}
	
	private Command receiveCommand() throws ClassNotFoundException, IOException {
		Command o = (Command) inStream.readObject();
		printScreen(o.toString(), "-->");
		return o;
	}

	public void sendCommand(Command o) throws IOException {
		outStream.writeObject(o);
		printScreen(o.toString(), "<--");
	}


	public void printScreen(String... data) {
		LocalDateTime t = LocalDateTime.now();
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss.SS");
		String formattedDate = t.format(formatter);
		String message = "";
		if (data.length == 1) {
			message = formattedDate + " | " + fillStringTo(" ", 28) + " |   | Server | " + data[0];
			System.out.println(message);
		} else if (data.length == 2) {
			String clientIP;
			if (data[1].equals("-->")) {
				clientIP = inSocket.getInetAddress().toString();
			} else {
				clientIP = outSocket.getInetAddress().toString();
			}
			message = formattedDate + " | " + fillStringTo(clientIP + " - " + name, 28) + " |" + data[1] + "| Server | " + data[0];
			System.out.println(message);
		}
		// --> IN | <-- OUT
	}
	
	public void printError(String data) {
		LocalDateTime t = LocalDateTime.now();
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss.SS");
		String formattedDate = t.format(formatter);
		String message = "";
		message = formattedDate + " | " + fillStringTo(" ", 28) + " |   | Server | " + data;
		System.err.println(message);
	}


	private String fillStringTo(String text, int count) {
		for (int i = text.length(); i <= count; i++) {
			text += " ";
		}
		return text.substring(0, count);
	}
}
